#!/usr/bin/env node
'use strict'
const path = require('path')
const os = require('os')

const inquirer = require('inquirer')
const chalk = require('chalk')
const updateNotifier = require('update-notifier')
const rimraf = require('rimraf')

const rc = require('../lib/rc')()
const pkg = require('../package.json')
const setup = require('./setup')
const selfAssessment = require('./selfAssessment')
const configPath = path.join(os.homedir(), '.adarc')
updateNotifier({pkg, updateCheckInterval: 1000 * 60 * 60}).notify()

const menu = () => {
  const ada = require('../lib/index')
  console.log(chalk.cyan.bold(`Welcome to ada-cli ${rc.get().medx.username}\n`))
  inquirer.prompt({
    type: 'list',
    name: 'home',
    message: 'Select an option to get started:',
    choices: [
      'Start Self Assessment',
      'Delete all profiles',
      'Logout'
    ]
  }, (answer) => {
    switch (answer.home) {
      case 'Start Self Assessment':
        return selfAssessment()
      case 'Delete all profiles':
        return ada('cli', {}, true, (err, success) => {
          if (err) return console.log(chalk.red.bold(`${err}\n`))
          console.log(chalk.green.bold(`${success}\n`))
          menu()
        })
      case 'Logout':
        rimraf(configPath, (err) => {
          if (err) return console.log(chalk.red.bold(err))
          console.log(chalk.green.bold('\nSuccessfully logged out.'))
        })
    }
  })
}

rc.configExists() ? menu() : setup(selfAssessment)
