'use strict'

const _ = require('lodash')
const request = require('request')
// require('request-debug')(request)
const attributeSelection = require('./attributeSelection')()

const config = require('./rc')().get()
const auth = config.medx.auth
const c3 = config.medx.c3.replace(/c3$/i, 'ada')
const OAUTH_BASICAUTH = 'Basic YWRhOjl4Q0dqZFJVcEJaVlVHdzJjVUROcTd2S3JCckFNQUU0'

const no = ['no', 'none', 'nope', 'nein']
const abortSearch = (term) => _.includes(no, term.trim().toLowerCase())

let currentProfile

const convertToDate = (date) => {
  const from = date.trim().split('.')
  return new Date(from[2], from[1] - 1, from[0], 12).toISOString()
}

module.exports = (username, password) => {
  let api = null
  const initialFormData = JSON.stringify({answer: {state: 'GET_HEALTH_ADVICE'}})
  const getResult = (props, cb) => {
    api.get({url: `result/${props.selfAssessmentKey}`}, (err, res, body) => {
      if (err) cb(err)
      cb(null, JSON.parse(body))
    })
  }

  const normalizeResponse = (response) => {
    const firstType = _.get(response, 'answerOptions[0].type')
    const secondType = _.get(response, 'answerOptions[1].type')

    response.answerType = firstType === 'SELECT' ? 'SELECT' : 'TEXT'
    response.search = firstType === 'SEARCH'

    if (firstType === 'SEARCH' && secondType) {
      response.question += ' Type "no" or enter a symptom.'
    }
    if (firstType === 'DATE') {
      response.question += ' e.g.: 19.07.1994'
    }
    if (firstType === 'ATTRIBUTE_SELECTION_MAP') {
      response.answerType = 'SELECT'
      response.answerOptions = attributeSelection.map((a) => {
        return _.defaults(a, response.answerOptions[0])
      })
    }
    return response
  }

  const symptomSearch = (answer, previousMessage, isCli, cb) => {
    const keys = previousMessage.answerOptions[0]
    const search = (sex, birthday) => {
      api.get({
        url: `/dialog_search?limit=20&offset=0&query=${encodeURIComponent(answer)}&` +
        `selfAssessmentKey=${keys.selfAssessmentKey}&sex=${sex}&birthday=${birthday}`
      },
      (err, res, body) => {
        let results = JSON.parse(body).results
        results.forEach((result) => {
          if (!result.name) return
          if (isCli) result.patientName += ` // medical: ${result.name}`
          if (!result.matchedName) return
          if (isCli) result.patientName += ` // matched: ${result.matchedName}`
        })
        const answerOptions = results.concat({
          key: 'backToSearch',
          name: '< back to search',
          patientName: '< back to search'
        })

        let newMessage = {
          question: 'Select a result:',
          answerType: 'SELECT',
          stateStack: previousMessage.stateStack,
          answerOptions: answerOptions,
          previousAnswerOptions: previousMessage.answerOptions,
          previousMessage
        }
        if (!results.length) console.log('No results were found.')
        cb(err, !err && results.length ? newMessage : previousMessage)
      })
    }
    if (!currentProfile || currentProfile.key !== keys.profileKey) {
      api.get({url: '/profiles'}, (err, res, rawProfiles) => {
        if (err) cb(err)
        const profiles = JSON.parse(rawProfiles)
        currentProfile = _.find(profiles, {key: keys.profileKey})
        search(currentProfile.sex, currentProfile.birthday)
      })
    } else {
      search(currentProfile.sex, currentProfile.birthday)
    }
  }

  const initiateChat = (cb) => {
    api.post({url: '/dialog', body: initialFormData}, (err, res, body) => {
      cb(err, !err && normalizeResponse(JSON.parse(body)))
    })
  }

  const deleteProfiles = (cb) => {
    api.del({url: '/profiles'}, (err, res, body) => {
      if (err) return cb(new Error(err))
      if (res.statusCode !== 204) return cb(new Error('Some error occured.'))
      cb(null, 'Successfully deleted all Profiles')
    })
  }

  return {
    login: (deleteAllProfiles, cb) => {
      const url = `${auth.replace(/api\/login$/i, '')}oauth/token?username=${encodeURIComponent(
          username)}&password=${encodeURIComponent(password)}&grant_type=password`
      request.post({url, headers: {
        Authorization: OAUTH_BASICAUTH
      }}, (err, res) => {
        if (err) return cb(new Error(err))
        const token = JSON.parse(res.body).access_token
        if (!token) return cb(new Error('login failed'))

        api = request.defaults({
          baseUrl: c3,
          headers: {
            Authorization: `Bearer ${token}`,
            'Content-Type': 'application/json;charset=UTF-8'
          }
        })
        if (deleteAllProfiles) return deleteProfiles(cb)
        initiateChat(cb)
      })
    },

    isLoggedIn: () => !!api,

    ask: (answer, previousMessage, isCli, cb) => {
      if (!api) cb(new Error('wrong function'))
      if (!previousMessage) return initiateChat(cb)

      if (answer.key === 'backToSearch') {
        return cb(null, previousMessage.previousMessage)
      }

      if (previousMessage.search && !abortSearch(answer)) {
        return symptomSearch(answer, previousMessage, isCli, cb)
      }

      let dialogFormData = {
        answer,
        stateStack: previousMessage.stateStack
      }

      if (previousMessage.answerType === 'TEXT') {
        const type = previousMessage.answerOptions[0].type
        const abortSearchAnswer = previousMessage.answerOptions[1] && previousMessage.answerOptions[1]

        answer = answer.trim()
        dialogFormData.answer = abortSearch(answer) ? abortSearchAnswer
        : {key: answer, input: type === 'DATE' ? convertToDate(answer) : answer}

        if (!abortSearchAnswer) {
          if (!dialogFormData.answer) return cb(null, previousMessage)
          dialogFormData.answer.state = previousMessage.answerOptions[0].state
        }

        dialogFormData.answer = _.defaults(dialogFormData.answer, previousMessage.answerOptions[0])
      }

      if (answer.score) {
        dialogFormData.answer.state = previousMessage.previousAnswerOptions[0].state
        dialogFormData.answer.input = answer.key
        Object.assign(dialogFormData.answer, previousMessage.previousAnswerOptions[0])
      }

      dialogFormData = JSON.stringify(dialogFormData)

      api.post({url: '/dialog', body: dialogFormData}, (err, res, body) => {
        body = JSON.parse(body)
        if (res.statusCode >= '300') return cb(new Error(res))
        if (!body.answerOptions) cb(new Error('No answer options were defined ', body))
        if (_.includes(['NAVIGATE', 'NAVIGATE_INSTANT'], body.answerOptions[0].type)) {
          if (body.answerOptions[0].route === 'assessmentReport') {
            return getResult(body.answerOptions[0].props, cb)
          }
          body.isOver = true
          return cb(null, body)
        }
        cb(err, !err && normalizeResponse(body))
      })
    }
  }
}
